from aws_cdk import Stack 
from constructs import Construct

from cdk_toolkit import cicd 


class ECRCodePipelineStack(Stack): 
    '''
    This Stack Creates a CodePipeline that Updates ECR Images when a given CodeCommit Repository is Updated
    '''
    def __init__(self, scope: Construct, construct_id: str, codecommit_repository_name: str, codecommit_repository_branch: str, codebuild_action_name: str, codebuild_role: str, codepipeline_name: str, codecommit_initial_commit_dir: str,  **kwargs) -> None:
        super().__init__(scope, construct_id, **kwargs)

        # Create CodeCommit Repository 
        codecommit_repository = cicd.createCodeCommitRepository(self, codecommit_repository_name, initial_commit_dir=codecommit_initial_commit_dir, initial_commit_branch=codecommit_repository_branch)

        # Create CodeCommit Artifact
        codecommit_artifact = cicd.createCodePipelineArtifact()

        # Create CodeCommit Source Action
        codecommit_source_action = cicd.createCodeCommitSourceAction(self, codecommit_repository, codecommit_repository_branch, codecommit_artifact)

        # Create CodeBuild Build Action 
        codebuild_build_action = cicd.createCodeBuildAction(self, codebuild_action_name, codecommit_artifact, codebuild_role)

        # Create CodePipeline
        codepipeline = cicd.createCodePipeline(self, codepipeline_name, codecommit_source_action, codebuild_build_action, deploy_action=None)


        


from aws_cdk import (  
    aws_codepipeline_actions as codepipeline_actions,
    aws_codebuild as codebuild
) 


##
## CodeBuild
def createCodeBuildAction(self, codebuild_action_name, codebuild_input_artifact, codebuild_role):
    """
    Creates a CodeBuild Action.
 
    :param codebuild_action_name: Name of the CodeBuild Action
    :param codebuild_input_artifact: CodeBuild Input Artifact (Source Output)
    :param codebuild_role: IAM CodeBuild Role Object
    :return: CodeBuild Build Action
    """ 
    codebuild_build_action = codepipeline_actions.CodeBuildAction(
        action_name=codebuild_action_name,
        # Configure your project here
        project=codebuild.PipelineProject(
            self, codebuild_action_name,
            project_name=codebuild_action_name,
            # role=codebuild_role,
            environment=codebuild.BuildEnvironment(
                privileged=True
            ), 
        ),
        input=codebuild_input_artifact,
    )
    return codebuild_build_action


from aws_cdk import ( 
    aws_codecommit as codecommit, 
    aws_codepipeline_actions as codepipeline_actions, 
) 



##
## CodeCommit
def createCodeCommitRepository(self, repository_name, initial_commit_dir=None, initial_commit_branch=None):
    """
    Creates a CodeCommit Repository.
 
    :param repository_name: Name of the CodeCommit Repository
    :param initial_commit: CodeCommit Repository Initial Commit (defaul=None)
    :return: CodeCommit Repository Object
    """ 
    code = None
    commit_branch = "main"
    if initial_commit_branch is None:
        commit_branch = initial_commit_branch
    if initial_commit_dir is not None:
        code = codecommit.Code.from_directory(initial_commit_dir, commit_branch)
    repo = codecommit.Repository(
        self, 'CDK-CodeCommit-Repository-{}'.format(repository_name),
        repository_name= repository_name,
        code=code
    )
    return repo

def existingCodeCommitRepository(self):
    return 

def createCodeCommitSourceAction(self, codecommit_repository, codecommit_repository_branch, codecommit_output_artifact):
    """
    Creates a CodeCommit Source Action.
 
    :param codecommit_repository: CodeCommit Repository Object
    :param codecommit_repository_branch: CodeCommit Repository Bran
    :param codecommit_output_artifact: CodeCommit Output Artifact
    :return: CodeCommit Source Action
    """ 
    codecommit_source_action = codepipeline_actions.CodeCommitSourceAction(
        action_name="Source", 
        repository=codecommit_repository,
        branch=codecommit_repository_branch, 
        output=codecommit_output_artifact 
    )  
    return codecommit_source_action


print("AWS CDK TOOLKIT")

from cdk_toolkit import (
    cicd, 
    connection, 
    container, 
    database, 
    datalake, 
    notebook, 
    notification, 
    permission, 
    serverless, 
    storage
) 


from constructs import Construct
from aws_cdk import (
    Aws, 
    RemovalPolicy, 
    aws_s3 as s3, 
)

##
## S3 Buckets
def createS3Bucket(self, bucket_name, versioned=False): 
    """
    Creates an S3 Bucket.
 
    :param bucket_name: Name of the S3 Bucket
    :param versioned: Enable Bucket Versioning (default=False)
    :return: S3 Bucket Object
    """ 
    bucket = s3.Bucket(
        self, "DataLakeS3Bucket",
        bucket_name=bucket_name,
        versioned=versioned,
        removal_policy=RemovalPolicy.DESTROY
    )
    return bucket

def existingS3Bucket():
    return 


##
## EFS


##
## EBS


# https://python-packaging-tutorial.readthedocs.io/en/latest/setup_py.html

from setuptools import setup, find_packages

setup(
    name='cdk-toolkit',
    version='0.0.15',
    author='Ryan Moos',
    author_email='ryan@moos.engineering',
    packages=find_packages(),
    # scripts=['bin/script1','bin/script2'],
    url='http://pypi.python.org/pypi/cdk-toolkit/',
    license='LICENSE',
    description='AWS Cloud Development Toolkit',
    long_description=open('README.md').read(),
    long_description_content_type="text/markdown",
    install_requires=[
        "aws-cdk-lib",
        "boto3",
    ],
)




from aws_cdk import (  
    aws_codepipeline as codepipeline, 
) 
 

##
## CodePipeline
def createCodePipelineArtifact():
    artifact = codepipeline.Artifact()
    return artifact

def createCodePipeline(self, codepipeline_name, source_action, build_action, deploy_action=None):
    stages = []
    stages.append(codepipeline.StageProps(stage_name="Source", actions=[source_action]))
    stages.append(codepipeline.StageProps(stage_name="Build", actions=[build_action]))
    if deploy_action is not None:
        stages.append(codepipeline.StageProps(stage_name="Deploy", actions=[deploy_action]))

    pipeline = codepipeline.Pipeline(
        self, "CDK-CodePipeline-{}".format(codepipeline_name),
        pipeline_name="{}".format(codepipeline_name), 
        stages=stages, 
    ) 
    return pipeline